#include <stdlib.h> // getenv()
#include <glib.h>
#include <glib/gi18n.h>
#include <gdk/gdk.h>
#include <gtk/gtk.h>
#include <assert.h>

#include "support.h"

void about_show_installation_info_cb(GtkWidget *widget, GdkEvent *event, gpointer data);
gchar *about_get_installation_details();

// Contributors should edit these variables
const gchar *AUTHORS[] = {"Osmo Antero Maatta [osmoma at gmail dot com]", NULL};
const gchar *DOCUMENTERS[] = {"", NULL};
const gchar *ARTISTS[] = {"", NULL};
const gchar *TRANSLATORS = 
    "Brazilian Portuguese (pt_BR): José Humberto Melo.\n"
	"Australian English (en_AU): Gary M.\n"
	"French (fr): Julien Nicolas, Quentin Theuret.\n"
	"German (de): Mario Blättermann.\n"
	"Hungarian (hu): Papp Bence.\n"
	"Norwegian Bokmål (nb_NO): Osmo Antero.\n"
	"Suomi/Finnish (fi_FI): Osmo Antero.\n";

const gchar *WEBSITE_URL = "https://launchpad.net/rec-applet";

static gchar *load_license_text(gchar *file)
{
	gchar *contents = NULL;
	gsize length;
	GError *error = NULL;

	if (!file) return NULL;

	gint ret = g_file_get_contents(file, &contents, &length, &error);
	if (error)
		g_error_free(error);

	// Did we find the license text?
	if (!ret || length <= 0)
	{
		g_free(contents);
		contents = g_strdup_printf("%s%s", _("This product is released under terms of GPL, GNU GENERAL PUBLIC LICENSE v3.\n"),
                                           _("Please see http://www.gnu.org/licenses/gpl-3.0.txt for more details."));
	}
	return contents;	
}

void about_this_app()
{
	GtkAboutDialog *dialog = (GtkAboutDialog*)gtk_about_dialog_new();

    // Show the translated application name
    gchar *package_name = _("Audio Recorder Applet");

	gtk_about_dialog_set_program_name(dialog, package_name);
	gtk_about_dialog_set_version(dialog, PACKAGE_VERSION); 
	gtk_about_dialog_set_copyright(dialog, "Osmo Antero Maatta, Norway, Oslo");

    // Show the English name "Audio Recorder Applet"
	gtk_about_dialog_set_comments(dialog, PACKAGE_NAME);

	// Show license text
    gchar *data_dir = get_data_directory();
	gchar *filename = g_build_filename(data_dir, "COPYING", NULL);
	gchar *text = load_license_text(filename);
	gtk_about_dialog_set_license(dialog, text);
	g_free(data_dir);
	g_free(filename);

	gtk_about_dialog_set_website(dialog, WEBSITE_URL);
	gtk_about_dialog_set_website_label(dialog, WEBSITE_URL);

	gtk_about_dialog_set_authors(dialog, AUTHORS);
	gtk_about_dialog_set_documenters(dialog, DOCUMENTERS);

	gtk_about_dialog_set_translator_credits(dialog, TRANSLATORS);
	gtk_about_dialog_set_artists(dialog, ARTISTS);

	// Add an extra button for "Installation Info"
	GtkWidget *hbutton_box = gtk_hbutton_box_new();
	gtk_widget_show(hbutton_box);
	gtk_container_add(GTK_CONTAINER(GTK_DIALOG(dialog)->vbox), hbutton_box);

	GtkWidget *button = gtk_button_new_with_label(_("Installation details"));
	gtk_widget_show(button);
	g_signal_connect((gpointer)button, "clicked", G_CALLBACK(about_show_installation_info_cb), NULL);
	gtk_container_add(GTK_CONTAINER(hbutton_box), button);

	gtk_dialog_run(GTK_DIALOG(dialog));
    gtk_widget_destroy(GTK_WIDGET(dialog));

	g_free(text);
}

void about_show_installation_info_cb(GtkWidget *widget, GdkEvent *event, gpointer data)
{
	/* Show dialog with installation details */

	GtkWidget *dialog = gtk_dialog_new_with_buttons(_("Installation details"),
		                                       NULL,
		                                       GTK_DIALOG_DESTROY_WITH_PARENT,
		                                       GTK_STOCK_OK,
		                                       GTK_RESPONSE_ACCEPT,
		                                       NULL);

		gtk_window_set_resizable(GTK_WINDOW(dialog), TRUE);
		gtk_window_set_default_size(GTK_WINDOW(dialog), 550, 450);
		gtk_dialog_set_default_response(GTK_DIALOG(dialog), GTK_RESPONSE_ACCEPT);

		GtkWidget *vbox0 = gtk_vbox_new(FALSE, 0);
		gtk_widget_show(vbox0);
		gtk_container_add(GTK_CONTAINER(GTK_DIALOG(dialog)->vbox), vbox0);

		GtkWidget *scrolledwindow = gtk_scrolled_window_new (NULL, NULL);
		gtk_widget_show(scrolledwindow);
		gtk_box_pack_start(GTK_BOX(vbox0), scrolledwindow, TRUE, TRUE, 0);
		gtk_scrolled_window_set_shadow_type(GTK_SCROLLED_WINDOW (scrolledwindow), GTK_SHADOW_IN);

		GtkWidget *text_field = gtk_text_view_new();
		gtk_widget_show(text_field);

		GtkTextBuffer *buffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW(text_field));

		gchar *text = about_get_installation_details();

		gtk_text_buffer_set_text (buffer, text,  -1);
	  	gtk_container_add(GTK_CONTAINER(scrolledwindow), text_field);

		gtk_dialog_run(GTK_DIALOG(dialog));
		gtk_widget_destroy(dialog);

		g_free(text);
}

gchar *about_get_installation_details()
{
	// Current language settings
	const gchar* const *lang_args = g_get_language_names();
	guint i = 0;
    GString *s = g_string_new(NULL);
	while (lang_args && lang_args[i])
	{
        g_string_append_printf(s, "%s, ", lang_args[i]);
		i++;
	}

	/*The array is owned by Glib. Do not free it.
	  g_strfreev(lang_args);
	*/

	// TODO: 
	// Should we translate these?
	gchar *text = g_strdup_printf(
    "Package name: %s\n"
    "Package version: %s\n" 
    "Installation prefix: %s\n" 
    "Binary location: %s  (executable name is: %s)\n"
    "Pixmap location: %s/\n"
    "Data location: %s/\n"  
    "Language locale directory: %s\n" 
    "Available languages are: %s\n"
    "LANG=%s\n"
    "\n"
    "Please send bug reports to: %s\n"
    "\n"
    "You can find other values in the GNOME's registry.\nStart gconf-editor and browse to /apps/audio_recorder_applet/\n",
    PACKAGE, 
    PACKAGE_VERSION,
    PREFIX, 
    PACKAGE_BIN_DIR, PACKAGE, 
    PIXMAPS_DIR, 
    PACKAGE_DATA_DIR, 
    PACKAGE_LOCALE_DIR, 
    s->str, 
    getenv("LANG"), 
    PACKAGE_BUGREPORT);

	g_string_free(s, TRUE);

	return text;
}


