#include "gconf.h"
#include "support.h"

// Write and read values from/to the GConf registry.
// Start: gconf-editor and bowse to /apps/audio_recorder_applet/.

#define APPLICATION_KEY "audio_recorder_applet" 

gchar *conf_get_app_name()
{	
	// Caller should g_free() this value
	return g_strdup(APPLICATION_KEY);
}

gchar *conf_get_key_path(gchar *key)
{
	gchar *app = conf_get_app_name();
	gchar *key_path = g_strdup_printf("/apps/%s/%s", app, key);
	g_free(app);
	return key_path;	
}

void conf_save_boolean_value(gchar *key, gboolean value) 
{
	GConfClient *gc_client = gconf_client_get_default();
	g_return_if_fail(GCONF_IS_CLIENT(gc_client));

	gchar *key_path = conf_get_key_path(key);

	if(!gconf_client_set_bool(gc_client, key_path, value, NULL)) 
	{
		g_warning("sd_gconf.c: Failed to set key %s (%s)\n", key_path, (value ? "True" : "False") );
	}

	g_free(key_path);
	g_object_unref(gc_client);
}

void conf_save_int_value(gchar *key, gint value) 
{
	GConfClient *gc_client = gconf_client_get_default();
	g_return_if_fail(GCONF_IS_CLIENT(gc_client));

	gchar *key_path = conf_get_key_path(key);

	if(!gconf_client_set_int(gc_client, key_path, value, NULL)) 
	{
		g_warning("sd_gconf.c: Failed to set key %s (%d)\n", key_path, value);
	}

	g_free(key_path);
	g_object_unref(gc_client);
}

void conf_save_string_value(gchar *key, gchar *value) 
{
	GConfClient *gc_client = gconf_client_get_default();
	g_return_if_fail(GCONF_IS_CLIENT(gc_client));

	gchar *key_path = conf_get_key_path(key);

	if(!gconf_client_set_string(gc_client, key_path, value, NULL)) 
	{
		g_warning("sd_gconf.c: Failed to set key %s (%s)\n", key_path, value);
	}

	g_free(key_path);
	g_object_unref(gc_client);
}

#if 0 
static void conf_free_value_list(GSList **list)
{
	GSList *sitem = *list;
	while (sitem)
	{
		GConfValue *value = sitem->data;
 		gconf_value_free(value);
		sitem = sitem->next;
	}
	g_slist_free(*list);
	*list = NULL;
}
#endif

void conf_save_string_list(gchar *key, GSList *list) 
{
	GConfClient *gc_client = gconf_client_get_default();
	g_return_if_fail(GCONF_IS_CLIENT(gc_client));

	gchar *key_path = conf_get_key_path(key);

	if(!gconf_client_set_list(gc_client, key_path, GCONF_VALUE_STRING, list, NULL)) 
	{
		g_warning("sd_gconf.c: Failed to save list to key %s.\n", key_path);
	}

	g_free(key_path);
	g_object_unref(gc_client);
}

gboolean conf_get_boolean_value(gchar *key, gboolean *value)
{  
	*value = FALSE;
	gboolean ret = FALSE;

	GConfClient *gc_client = gconf_client_get_default();
	g_return_val_if_fail(GCONF_IS_CLIENT(gc_client), FALSE);

	gchar *key_path = conf_get_key_path(key);

	GConfValue *gc_val = NULL;    
	gc_val = gconf_client_get_without_default(gc_client, key_path, NULL);

	/* If value pointer remains NULL, the key was not found */
	if(gc_val == NULL) 
	{
		g_warning("sd_conf.c: Cannot find key %s.\n", key_path);
		goto l_end;
	}
    
	/* Check if the value type is integer */
	if(gc_val->type == GCONF_VALUE_BOOL) 
	{
		*value = gconf_value_get_bool(gc_val);
	}

	ret = TRUE;

l_end:
	g_free(key_path);
	if (gc_val) 
		gconf_value_free(gc_val);
	g_object_unref(gc_client);
	return ret;
}
 
gboolean conf_get_int_value(gchar *key, gint *value)
{  
	*value = 0L;
	gboolean ret = FALSE;

	GConfClient *gc_client = gconf_client_get_default();
	g_return_val_if_fail(GCONF_IS_CLIENT(gc_client), FALSE);

	gchar *key_path = conf_get_key_path(key);

	GConfValue *gc_val = NULL;    
	gc_val = gconf_client_get_without_default(gc_client, key_path, NULL);

	if(gc_val == NULL) 
	{
		g_warning("sd_conf.c: Cannot find key %s.\n", key_path);
		goto l_end;
	}
    
	/* Check if the value type is integer */
	if(gc_val->type == GCONF_VALUE_INT) 
	{
		*value = gconf_value_get_int(gc_val);
	}
	ret = TRUE;

l_end:
	g_free(key_path);
	if (gc_val) 
		gconf_value_free(gc_val);
	g_object_unref(gc_client);
	return ret;
}

gboolean conf_get_string_value(gchar *key, gchar **value)
{  
	*value = NULL;
	gboolean ret = FALSE;

	GConfClient *gc_client = gconf_client_get_default();
	g_return_val_if_fail(GCONF_IS_CLIENT(gc_client), FALSE);

	gchar *key_path = conf_get_key_path(key);

	GConfValue *gc_val = NULL;    
	gc_val = gconf_client_get_without_default(gc_client, key_path, NULL);

	if(gc_val == NULL) 
	{
		g_warning("sd_conf.c: Cannot find key %s.\n", key_path);
		goto l_end;
	}
    
	/* Check if the value type is string */
	if(gc_val->type == GCONF_VALUE_STRING) 
	{
		*value = g_strdup(gconf_value_get_string(gc_val));
	}
	ret = TRUE;

l_end:
	g_free(key_path);    
	if (gc_val) 
		gconf_value_free(gc_val);
	g_object_unref(gc_client);
	return ret;
}

gboolean conf_get_string_list(gchar *key, GSList **list) 
{
	*list = NULL;
	gboolean ret = FALSE;

	GConfClient *gc_client = gconf_client_get_default();
	g_return_val_if_fail(GCONF_IS_CLIENT(gc_client), FALSE);

	gchar *key_path = conf_get_key_path(key);

	*list = gconf_client_get_list(gc_client, key_path, GCONF_VALUE_STRING, NULL);

	if (!(*list))
	{
		g_warning("sd_gconf.c: Failed to get list for key %s.\n", key_path);
		goto l_end;
	}
	ret = TRUE;

l_end:
	g_object_unref(gc_client);
	g_free(key_path);

	return ret;
}


