#ifndef _GST_RECORDER_H
#define _GST_RECORDER_H

#include <glib.h>
#include <gdk/gdk.h>
#include <glib/gstdio.h>
#include <gst/gst.h>
#include <gst/interfaces/propertyprobe.h>

// Number of level/amplitude (dB) values to collect
#define MAX_LEVEL_VALUES 20

// Data from the GStreamer's level element
typedef struct {
	// Ref: http://www.gstreamer.net/data/doc/gstreamer/head/gst-plugins-good-plugins/html/gst-plugins-good-plugins-level.html
	gdouble rms_dB;
	gdouble peak_dB;
	gdouble norm_peak; // normalized peak_DB = exp(peak_dB / 20), [0.0  - 1.0] 
	gdouble decay_dB;
} LevelData;

typedef struct {
	// GStreamer pipeline
	GstElement *pipeline;

	// Tick function (id from g_timeout_add_seconds_full())
	// Managed by gst_recorder.c
	guint tick_func_id; 

	// Stop-timer function (id from g_timeout_add_seconds_full())
	// Managed by timer.c
	guint timer_func_id;	

	// Lock on data 
	GMutex *data_lock;

 	 // These variables are controlled by the lock

	 // Circular buffer of amplitude/level values from GStreamer's level element.
	 // Values are used to test silence or talk/music.
	 LevelData level_buf[MAX_LEVEL_VALUES];
	 guint level_buf_pos;

} GstRec;

void rec_set_data_lock();
void rec_free_data_lock();

void gst_recorder_init();
void gst_recorder_exit();

gint64 gst_get_stream_time();

gboolean gst_recorder_is_recording();

gboolean gst_recorder_start_recording();
gboolean gst_recorder_stop_and_reset();
gboolean gst_recorder_stop_recording();

gchar *gst_get_output_filename();

#endif
