#include "applet.h"
#include "gconf.h"
#include "support.h"
#include "utility.h"
#include "gst-recorder.h"
#include "audio-sources.h"
#include "rec-window.h"
#include "media-profiles.h"
#include "help.h"

// The settings window, and all its widgets. Global variable.
SettingsWindow g_win;

// Reference to the applet record (and widgets on the panel/toolbar)
extern AppletRec g_applet;

extern void applet_set_widgets();

static void win_save_values();
static void win_set_position();

void win_close_button_cb(GtkButton *button, gpointer user_data) {
	if (GTK_IS_WINDOW(g_win.window)) {

        // Let's save the values here (unfortunately we do not have a [Cancel] button ATM)
        win_save_values();

		gtk_widget_destroy(g_win.window);
	}
	g_win.window = NULL;
}

void win_start_recording_cb(GtkButton *button, gpointer user_data) {
	// Start or stop recording 

	// Already recording?
	gboolean recording = gst_recorder_is_recording();

	if (!recording) {
		// Save values in GConf registry
		win_save_values();
		// Start recording
		recording = gst_recorder_start_recording();
	}
	else {
		// Stop recording
		gst_recorder_stop_recording();
		recording = FALSE;
	}

	// Set button label and image
	win_set_widgets();

	// Show the generated/replaced filename
	if (recording && GTK_IS_WINDOW(g_win.window) && GTK_IS_WIDGET(g_win.filename)) {
		// Show the real, final filename on the display
	
		gchar *last_filename = NULL;
		conf_get_string_value("last_filename", &last_filename);
		gtk_entry_set_text(GTK_ENTRY(g_win.filename), last_filename);
		g_free(last_filename);

		// Start with blank values
		gtk_label_set_text(GTK_LABEL(g_win.time_label), "00:00:00");
		gtk_label_set_text(GTK_LABEL(g_win.size_label), "0.0 KB");
	}
}

void win_save_values() {
	if (!GTK_IS_WINDOW(g_win.window)) {
		return;
	}

	// Save file name
	gchar *txt = (gchar*)gtk_entry_get_text(GTK_ENTRY(g_win.filename));
	conf_save_string_value("last_filename", txt);

	gboolean b = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(g_win.add_to_file)); 	
	conf_save_boolean_value("append_to_file", b);

	// Values changed?
	gboolean changed = FALSE;

	// Save timer values
	b = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(g_win.timer_active)); 	
	gboolean b2 = FALSE;
	conf_get_boolean_value("timer_active", &b2);
	if (b != b2) {	
		conf_save_boolean_value("timer_active", b);
		changed = TRUE;	
	}

	txt = gtk_combo_box_get_active_text(GTK_COMBO_BOX(g_win.timer_type));
	gchar *txt2 = NULL;
	conf_get_string_value("timer_type", &txt2);

	if (g_strcmp0(txt, txt2)) {
		conf_save_string_value("timer_type", txt);
		changed = TRUE;
	}
	g_free(txt2);

	// txt = (gchar*)gtk_entry_get_text(GTK_ENTRY(g_win.timer_value));
	txt = gtk_combo_box_get_active_text(GTK_COMBO_BOX(g_win.timer_value));
	conf_get_string_value("timer_value", &txt2);

	if (g_strcmp0(txt, txt2)) {
		conf_save_string_value("timer_value", txt);
		changed = TRUE;
	}
	g_free(txt);
	g_free(txt2);

	if (changed) {
		// Increment the counter so the timer function knows the values have been altered
		gint count = 0;
		conf_get_int_value("timer_setting_counter", &count);
		conf_save_int_value("timer_setting_counter", count+1);
	}

	// Save audio device/source
	txt = gtk_combo_box_get_active_text(GTK_COMBO_BOX(g_win.audio_device));
	conf_save_string_value("audio_device", txt);
	g_free(txt);

    // Save media-profile (id)
    txt = media_profiles_get_selected_id(g_win.media_format);
	conf_save_string_value("media_format", txt);
	g_free(txt);

    // Folder name
	txt = (gchar*)gtk_entry_get_text(GTK_ENTRY(g_win.folder_name));
	conf_save_string_value("folder_name", txt);

    // Filename pattern
	txt = (gchar*)gtk_entry_get_text(GTK_ENTRY(g_win.filename_pattern));
	conf_save_string_value("filename_pattern", txt);
}

void combo_select_string(GtkComboBox *combo, gchar *str, gint sel_row) {
    // Find str in the GtkComboBox (combo).
    // If str is not found then select sel_row line.
    // If sel_row < 0, then add a new row (with str) and select it.
	GtkTreeModel *model = gtk_combo_box_get_model(combo);
	GtkTreeIter iter;

    // Loop for all rows
	gint ret = gtk_tree_model_get_iter_first(model, &iter);
	while (ret && str)
	{
		gchar *val = NULL;
		gtk_tree_model_get(model, &iter, 0, &val, -1);

		// Strings match?
		if (strcmp(val, str) == 0) {
			// Select this item
			gtk_combo_box_set_active_iter(combo, &iter);
			g_free(val);
            // And quit
			return;
		}
		g_free(val);
		ret = gtk_tree_model_iter_next(model, &iter);
	}	

    // Item was not found. 
    // Select sel_row. If sel_row < 0, then add a new row and select it.
    if (sel_row >= 0) {
	    // Select sel_row
	    gtk_combo_box_set_active(combo, sel_row);

    } else {
        // Add a new row and select it
        gtk_combo_box_prepend_text(GTK_COMBO_BOX(combo), str);
        gtk_combo_box_set_active(GTK_COMBO_BOX(combo), 0);
    }
}

static void win_expander_click_cb(GObject *object, GParamSpec *param_spec, gpointer userdata) {
	GtkExpander *expander = GTK_EXPANDER(object);
	gboolean expanded = gtk_expander_get_expanded(expander);

	gchar *expander_name = (gchar*)userdata;

	// Save the expanded status
	conf_save_boolean_value(expander_name, expanded);
}

void get_folder_name(GtkWidget *button, gpointer user_data) {

	GtkWidget *entry = (GtkWidget*)user_data;

	GtkWidget *dialog;

	// Directory chooser dialog, select directory
	dialog = gtk_file_chooser_dialog_new(_("Select Directory"), 
		                                GTK_WINDOW (g_win.window),
		                                GTK_FILE_CHOOSER_ACTION_SELECT_FOLDER,
		                                GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
		                                GTK_STOCK_OPEN, GTK_RESPONSE_ACCEPT,
		                                NULL);
  
	gchar *path = g_strdup((gchar*)gtk_entry_get_text(GTK_ENTRY(entry)));
	if (str_length(path, 2048) < 1) {
		g_free(path);
		path = g_strdup(getenv("HOME"));
	}

	gtk_file_chooser_set_current_folder(GTK_FILE_CHOOSER(dialog), path);

	guint response = gtk_dialog_run(GTK_DIALOG(dialog));

	g_free(path); path = NULL;

	switch (response)
	{
		case GTK_RESPONSE_ACCEPT:
		{
			path = gtk_file_chooser_get_filename(GTK_FILE_CHOOSER (dialog));
		}
	}
  
	gtk_widget_destroy(dialog);

	if (path) {
		gtk_entry_set_text(GTK_ENTRY(entry), path);
		g_free(path);
	}
}

void win_set_widgets() {
	// Are we recording?
	gboolean recording = gst_recorder_is_recording();

	gchar *label = NULL;
	gchar *image_file = NULL;

	if (recording) {
		// Red circle (means ON)
		image_file = (gchar*)get_image_path("recording-on-small.png");

		// Button label
		label = _("Sto_p Recording");
	} else {
		// Green circle (means OFF)
		image_file = (gchar*)get_image_path("recording-off-small.png");

		// Button label
		label = _("_Start Recording");
	}

	// Set label
	if (GTK_IS_BUTTON(g_win.recorder_button)) {
		gtk_button_set_label(GTK_BUTTON(g_win.recorder_button), label); 
		// Set image
		GtkWidget *image = gtk_image_new_from_file(image_file);  
		gtk_button_set_image(GTK_BUTTON(g_win.recorder_button), image);

	}
	g_free(image_file);

	// Update also applet image on the toolbar
	applet_set_widgets();

	// Reset amplitude/level bar
	if (!recording && GTK_IS_PROGRESS_BAR(g_win.level_bar)) {
		gtk_progress_bar_set_fraction(GTK_PROGRESS_BAR(g_win.level_bar), 0.0);
	}
}

void window_hide() {
	if (GTK_IS_WINDOW(g_win.window)) {
		win_save_values();
		gtk_widget_destroy(g_win.window);
		g_win.window = NULL;
	}
}

void win_timer_active_cb(GtkToggleButton *togglebutton, gpointer user_data) {
	// Timer active checkbox clicked
	gboolean active = gtk_toggle_button_get_active(togglebutton);

	if (GTK_IS_WIDGET(g_win.timer_type)) {
		gtk_widget_set_sensitive(g_win.timer_type, active);
		gtk_widget_set_sensitive(g_win.timer_value, active);
	}
}

void window_show_timer_help() {
    // Show help file
    help_show_page("timer-syntax.html");
}

void window_show_filename_help() {
    // Show help file
    help_show_page("filename-format.html");
}

void window_create()
{
	if (GTK_IS_WINDOW(g_win.window)) {
		window_hide();
		return;
	}

	g_win.window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
	gtk_window_stick(GTK_WINDOW(g_win.window));
	gtk_window_set_skip_taskbar_hint(GTK_WINDOW(g_win.window), TRUE);
	gtk_window_set_skip_pager_hint(GTK_WINDOW(g_win.window), TRUE);

	gtk_window_set_decorated(GTK_WINDOW(g_win.window), FALSE);

	gtk_window_set_default_size(GTK_WINDOW(g_win.window), PREF_WINDOW_WIDTH, -1);

	GtkWidget *frame = gtk_frame_new(NULL);
	gtk_frame_set_shadow_type(GTK_FRAME(frame), GTK_SHADOW_IN);
	gtk_container_add(GTK_CONTAINER(g_win.window), frame);
	gtk_container_set_border_width(GTK_CONTAINER(frame), 7);

	GtkWidget *vbox0 = gtk_vbox_new (FALSE, 1);
	gtk_widget_show(vbox0);
	gtk_container_add(GTK_CONTAINER(frame), vbox0);

	GtkWidget *vbox1 = gtk_vbox_new (FALSE, 3);
	gtk_widget_show(vbox1);
	gtk_box_pack_start(GTK_BOX(vbox0), vbox1, FALSE, TRUE, 0);

	GtkWidget *hbox0 = gtk_hbox_new (FALSE, 0);
	gtk_widget_show(hbox0);
	gtk_box_pack_start(GTK_BOX(vbox1), hbox0, TRUE, TRUE, 0);

	// Get saved values
	gchar *last_filename = NULL;
	conf_get_string_value("last_filename", &last_filename);

	gchar *folder_name = NULL;
	conf_get_string_value("folder_name", &folder_name);

	gboolean append_to_file = FALSE;
	conf_get_boolean_value("append_to_file", &append_to_file);

	g_win.recorder_button = gtk_button_new_with_mnemonic("");
	gtk_box_pack_start(GTK_BOX (hbox0), g_win.recorder_button, FALSE, FALSE, 0);
	g_signal_connect(g_win.recorder_button, "clicked", G_CALLBACK(win_start_recording_cb), NULL);
	gtk_button_set_use_underline(GTK_BUTTON(g_win.recorder_button), TRUE);

	g_win.time_label = gtk_label_new("00:00:00");
	gtk_box_pack_start(GTK_BOX(hbox0), g_win.time_label, FALSE, FALSE, 2);
	gtk_widget_set_sensitive(g_win.time_label, FALSE);

	g_win.size_label = gtk_label_new("0.0 KB");
	gtk_box_pack_start(GTK_BOX(hbox0), g_win.size_label, FALSE, FALSE, 7);
	gtk_widget_set_sensitive(g_win.size_label, FALSE);

	// Show file size
	gchar *fname = g_build_filename(folder_name, last_filename, NULL);
	
	if (g_file_test(fname, G_FILE_TEST_EXISTS))
	{
		// Get the file size
		struct stat fstat;
		if (!stat(fname, &fstat)) {
			gchar *size_txt = format_file_size(fstat.st_size);
			gtk_label_set_text(GTK_LABEL(g_win.size_label), size_txt);
			g_free(size_txt);
		}
	}
	g_free(fname);

	GtkWidget *alignment0 = gtk_alignment_new(0.5, 0.5, 1, 1);
	gtk_box_pack_start(GTK_BOX(hbox0), alignment0, TRUE, TRUE, 0);
	gtk_widget_set_size_request(alignment0, -1, 9);
	gtk_alignment_set_padding(GTK_ALIGNMENT(alignment0), 10, 10, 0, 0);

	g_win.level_bar = gtk_progress_bar_new();
	gtk_container_add(GTK_CONTAINER (alignment0), g_win.level_bar);
	gtk_progress_bar_set_fraction(GTK_PROGRESS_BAR(g_win.level_bar), 0.0);

	GtkWidget *hbox1 = gtk_hbox_new(FALSE, 0);
	gtk_box_pack_start(GTK_BOX(vbox1), hbox1, TRUE, TRUE, 0);

	// "File name" label
	GtkWidget *label0 = gtk_label_new(_("File name:"));
	gtk_box_pack_start(GTK_BOX(hbox1), label0, FALSE, FALSE, 0);
	gtk_misc_set_alignment(GTK_MISC(label0), 0.01, 0.5);

	g_win.filename = gtk_entry_new();
	gtk_box_pack_start(GTK_BOX(hbox1), g_win.filename, TRUE, TRUE, 0);
	gtk_entry_set_invisible_char(GTK_ENTRY(g_win.filename), 9679);

	// Show last_filename; basename.ext
	gtk_entry_set_text(GTK_ENTRY(g_win.filename), last_filename);

	// Free last_filename
	g_free(last_filename);

	// "Add" to file checkbox label
	g_win.add_to_file = gtk_check_button_new_with_mnemonic(_("Add."));
	gtk_box_pack_start(GTK_BOX(hbox1), g_win.add_to_file, FALSE, FALSE, 0);
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(g_win.add_to_file), append_to_file); 	
	// -------

	// A simple timer/clock interface
	gchar *str_value = NULL;
	gboolean bool_value = FALSE;

	// "Timer>" GUI expander
	GtkWidget *timer_expander = gtk_expander_new(_("Timer."));
	gtk_box_pack_start(GTK_BOX(vbox1), timer_expander, TRUE, TRUE, 0);
	conf_get_boolean_value("timer_expanded", &bool_value);
	gtk_expander_set_expanded(GTK_EXPANDER(timer_expander), bool_value);
	g_signal_connect(timer_expander, "notify::expanded", G_CALLBACK(win_expander_click_cb), "timer_expanded");

	GtkWidget *hbox2 = gtk_hbox_new(FALSE, 0);
	gtk_container_add(GTK_CONTAINER(timer_expander), hbox2);

	g_win.timer_active = gtk_check_button_new();
	gtk_box_pack_start(GTK_BOX(hbox2), g_win.timer_active, FALSE, FALSE, 0);
	g_signal_connect(g_win.timer_active, "toggled", G_CALLBACK(win_timer_active_cb), NULL);

	conf_get_boolean_value("timer_active", &bool_value);
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(g_win.timer_active), bool_value); 	

	g_win.timer_type = gtk_combo_box_new_text();
	gtk_box_pack_start(GTK_BOX(hbox2), g_win.timer_type, FALSE, TRUE, 0);

	// "Stop after", stop recording after time duration | file size
	gtk_combo_box_append_text(GTK_COMBO_BOX(g_win.timer_type), _("Stop after")); // eg. "1 h(our) 5 m(inutes)"

	// "Stop at", stop recording at clock time | "silence"
	gtk_combo_box_append_text(GTK_COMBO_BOX(g_win.timer_type), _("Stop at"));  // eg. "10:30 pm"

	conf_get_string_value("timer_type", &str_value);
	combo_select_string(GTK_COMBO_BOX(g_win.timer_type), str_value, 0/*select 0'th row if str_value not found*/);
	g_free(str_value);
	gtk_widget_set_sensitive(g_win.timer_type, bool_value);

	// GtkComboBox
	g_win.timer_value = gtk_combo_box_entry_new_text();

	// Add some examples 
    {
	    gtk_combo_box_append_text(GTK_COMBO_BOX(g_win.timer_value), "silence");

	    // Stop recording after "silence"
	    if (g_strcmp0("silence", _("silence")) != 0) {
		    // Translated, localized text
		    gtk_combo_box_append_text(GTK_COMBO_BOX(g_win.timer_value), _("silence"));
	    }

        // Translators: 
	    // These strings h,m,s must start with a unique letter or sub-string. 
	    // "h)our".
	    gchar *hour_txt = _("h");  
	    // "m)inute"
	    gchar *minute_txt =  _("m");
	    // "s)econds"
	    gchar *seconds_txt =  _("s");

        // Show translated text of "2h 15m 10s" (if different from english)
	    if (g_strcmp0("h", hour_txt) ||
            g_strcmp0("m", minute_txt) ||  
            g_strcmp0("s", seconds_txt)) {

            gchar *txt = g_strdup_printf("2%s 15%s 10%s", hour_txt, minute_txt, seconds_txt);
            gtk_combo_box_append_text(GTK_COMBO_BOX(g_win.timer_value), txt);
            g_free(txt);
	    }

	    gtk_combo_box_append_text(GTK_COMBO_BOX(g_win.timer_value), "1h 5m 3s");

	    gtk_combo_box_append_text(GTK_COMBO_BOX(g_win.timer_value), "30 min | 1.5 GB | silence 10s");
	    gtk_combo_box_append_text(GTK_COMBO_BOX(g_win.timer_value), "14:30:15");
	    gtk_combo_box_append_text(GTK_COMBO_BOX(g_win.timer_value), "08:00");
	    gtk_combo_box_append_text(GTK_COMBO_BOX(g_win.timer_value), "07:00 pm");
	    gtk_combo_box_append_text(GTK_COMBO_BOX(g_win.timer_value), "1000 KB");
	    gtk_combo_box_append_text(GTK_COMBO_BOX(g_win.timer_value), "3.2 MB");
	    gtk_combo_box_append_text(GTK_COMBO_BOX(g_win.timer_value), "500 MB | 20 min");
    }

	gtk_box_pack_start(GTK_BOX(hbox2), g_win.timer_value, TRUE, TRUE, 0);
	gtk_widget_set_sensitive(g_win.timer_value, bool_value);

	GtkWidget *button0 = gtk_button_new();
	GtkWidget *image = gtk_image_new_from_stock("gtk-info", GTK_ICON_SIZE_BUTTON);
	gtk_button_set_image(GTK_BUTTON(button0), image); 
	g_signal_connect(button0, "clicked", G_CALLBACK(window_show_timer_help), NULL);
	gtk_box_pack_start(GTK_BOX(hbox2), button0, FALSE, FALSE, 0);

    // Show the current value
	conf_get_string_value("timer_value", &str_value);
	combo_select_string(GTK_COMBO_BOX(g_win.timer_value), str_value, -1/*add new row if str_value not found*/);
	g_free(str_value);

	GtkWidget *hseparator0 = gtk_hseparator_new();
	gtk_box_pack_start(GTK_BOX(vbox0), hseparator0, FALSE, TRUE, 0);

	// "Settings>" GUI expander
	GtkWidget *setting_expander = gtk_expander_new(_("Settings."));
	gtk_box_pack_start(GTK_BOX(vbox0), setting_expander, TRUE, TRUE, 2);
	conf_get_boolean_value("setting_expanded", &bool_value);
	gtk_expander_set_expanded(GTK_EXPANDER(setting_expander), bool_value);
	g_signal_connect(setting_expander, "notify::expanded", G_CALLBACK(win_expander_click_cb), "setting_expanded");

	GtkWidget *vbox2 = gtk_vbox_new (FALSE, 3);
	gtk_container_add(GTK_CONTAINER(setting_expander), vbox2);

	GtkWidget *table0 = gtk_table_new(5, 3, FALSE);
	gtk_box_pack_start(GTK_BOX (vbox2), table0, FALSE, FALSE, 0);
	gtk_table_set_row_spacings(GTK_TABLE(table0), 3);

	// "Audio source" label
	label0 = gtk_label_new(_("Audio source:"));
	gtk_table_attach (GTK_TABLE(table0), label0, 0, 1, 0, 1, (GtkAttachOptions) (GTK_FILL), (GtkAttachOptions) (0), 0, 0);
	gtk_misc_set_alignment(GTK_MISC(label0), 0, 0.5);

	g_win.audio_device = gtk_combo_box_new_text();
	gtk_table_attach(GTK_TABLE(table0), g_win.audio_device, 1, 3, 0, 1,(GtkAttachOptions) (GTK_EXPAND | GTK_FILL), (GtkAttachOptions) (GTK_FILL), 0, 0);

	// Get list of audio devices
	GList *lst = get_audio_devices(DEFAULT_AUDIO_SOURCE);
	GList *n = g_list_first(lst);
	while (n) {
		DeviceItem *item = (DeviceItem*)n->data;
		gchar *txt = item->name;

		// Show internal device id if the name is NULL (should not happend)
		if (txt == NULL)
			txt = item->id;

		gtk_combo_box_append_text(GTK_COMBO_BOX(g_win.audio_device), txt);

		n = g_list_next(n);
	}

	// Free the list
	g_list_foreach(lst, (GFunc)device_item_free, NULL);
	g_list_free(lst); lst = NULL;

	// Set/show the current value
	conf_get_string_value("audio_device", &str_value);
	combo_select_string(GTK_COMBO_BOX(g_win.audio_device), str_value, 0/*select 0'th row if str_value not found*/);
	g_free(str_value);

	// Audio format; .OGG, .MP3, .WAV, etc. See media-profiles.c
	label0 = gtk_label_new (_("Audio format:"));
	gtk_table_attach(GTK_TABLE(table0), label0, 0, 1, 1, 2, (GtkAttachOptions) (GTK_FILL), (GtkAttachOptions) (0), 0, 0);
	gtk_misc_set_alignment(GTK_MISC(label0), 0, 0.5);

    // Get the media-profiles combobox filled with values.
    // Start gconf-editor and browse to: system -> gstreamer -> 0.10 -> audio -> profiles.
	g_win.media_format = media_profiles_create_combobox();
	gtk_table_attach(GTK_TABLE(table0), g_win.media_format, 1, 2, 1, 2, (GtkAttachOptions) (GTK_FILL), (GtkAttachOptions) (GTK_FILL), 0, 0);

	// Set/show current value
	conf_get_string_value("media_format", &str_value);
	combo_select_string(GTK_COMBO_BOX(g_win.media_format), str_value, 0/*select 0'th row if str_value not found*/);
	g_free(str_value);

	hseparator0 = gtk_hseparator_new ();
	gtk_table_attach(GTK_TABLE(table0), hseparator0, 1, 2, 2, 3, (GtkAttachOptions) (GTK_FILL), (GtkAttachOptions) (GTK_FILL), 0, 0);

	hseparator0 = gtk_hseparator_new ();
	gtk_table_attach(GTK_TABLE(table0), hseparator0, 2, 3, 2, 3, (GtkAttachOptions) (GTK_FILL), (GtkAttachOptions) (GTK_FILL), 0, 0);


	// "Folder name:" label
	label0 = gtk_label_new(_("Folder name:"));
	gtk_table_attach (GTK_TABLE(table0), label0, 0, 1, 3, 4, (GtkAttachOptions) (GTK_FILL), (GtkAttachOptions) (0), 0, 0);
	gtk_misc_set_alignment(GTK_MISC(label0), 0, 0.5);

	g_win.folder_name = gtk_entry_new();
	gtk_table_attach(GTK_TABLE(table0), g_win.folder_name, 1, 2, 3, 4, (GtkAttachOptions) (GTK_EXPAND | GTK_FILL), (GtkAttachOptions) (0), 0, 0);
	gtk_entry_set_invisible_char(GTK_ENTRY (g_win.folder_name), 9679);

	button0 = gtk_button_new_with_mnemonic("...");
	gtk_table_attach(GTK_TABLE(table0), button0, 2, 3, 3, 4, (GtkAttachOptions) (GTK_FILL), (GtkAttachOptions) (0), 0, 0);
	g_signal_connect(button0, "clicked", G_CALLBACK(get_folder_name), g_win.folder_name);

	// Has folder name?
	if (str_length(folder_name, 1024) < 1) {
		g_free(folder_name);
		// Set default folder name; "/home/username/Audio"

		// Get $HOME, do not g_free() this
		gchar *home = getenv("HOME");

		// If user has ~/Music folder, then use the english word.
		gchar *path = g_strdup_printf("%s/%s", home, "Music");

		if (g_file_test(path, G_FILE_TEST_IS_DIR)) 
			folder_name = g_strdup_printf("%s/%s", home, "Audio");
		else
            // This is a folder name like "/home/username/Audio".
			folder_name = g_strdup_printf("%s/%s", home, _("Audio"));

		g_free(path);

		// g_mkdir(folder_name, 0755/* rwxr-xr-x */);
	}

	gtk_entry_set_text(GTK_ENTRY(g_win.folder_name), folder_name);
	g_free(folder_name);

	// "Filename format:" label
	label0 = gtk_label_new(_("Filename format:"));
	gtk_table_attach(GTK_TABLE(table0), label0, 0, 1, 4, 5, (GtkAttachOptions) (GTK_FILL), (GtkAttachOptions) (0), 0, 0);
	gtk_misc_set_alignment(GTK_MISC(label0), 0, 0.5);

	g_win.filename_pattern = gtk_entry_new();
	gtk_table_attach(GTK_TABLE(table0), g_win.filename_pattern, 1, 2, 4, 5, (GtkAttachOptions) (GTK_EXPAND | GTK_FILL), (GtkAttachOptions) (0), 0, 0);
	gtk_entry_set_invisible_char(GTK_ENTRY (g_win.filename_pattern), 9679);

	button0 = gtk_button_new();
	image = gtk_image_new_from_stock("gtk-info", GTK_ICON_SIZE_BUTTON);
	gtk_button_set_image(GTK_BUTTON(button0), image); 
	g_signal_connect(button0, "clicked", G_CALLBACK(window_show_filename_help), NULL);
	gtk_table_attach (GTK_TABLE (table0), button0, 2, 3, 4, 5, (GtkAttachOptions) (GTK_FILL), (GtkAttachOptions) (0), 0, 0);

	// Set current value
	conf_get_string_value("filename_pattern", &str_value);
	if (str_length(str_value, 128) < 2) {
		g_free(str_value);
		// Default filename pattern, "rec" means "recording".
		str_value = g_strdup(_("rec-%C%y%m%d-%H:%M:%S"));
	}

	gtk_entry_set_text(GTK_ENTRY(g_win.filename_pattern), str_value);
	g_free(str_value);

	hseparator0 = gtk_hseparator_new ();
	gtk_box_pack_start(GTK_BOX(vbox0), hseparator0, FALSE, TRUE, 0);

	GtkWidget *hbox3 = gtk_hbox_new(FALSE, 0);
	gtk_box_pack_start(GTK_BOX(vbox0), hbox3, FALSE, TRUE, 0);

	button0 = gtk_button_new_from_stock("gtk-close");
	gtk_box_pack_end(GTK_BOX(hbox3), button0, FALSE, FALSE, 0);
	g_signal_connect(button0, "clicked", G_CALLBACK(win_close_button_cb), NULL);

	gtk_widget_show_all(g_win.window);

	// Set/reset labels and values
	win_set_widgets();

	// Set window's position
	win_set_position(g_win.window);
}

static void win_set_position() {
	gint x, y, width, height;
	GtkRequisition  req;

	gtk_widget_size_request(GTK_WIDGET(g_win.window), &req);
	gdk_window_get_origin(GTK_WIDGET(g_applet.applet)->window, &x, &y);
	gdk_window_get_geometry(GTK_WIDGET(g_applet.applet)->window, NULL, NULL, &width, &height, NULL);

	switch (panel_applet_get_orient(g_applet.applet)) {
		case PANEL_APPLET_ORIENT_DOWN:
		y += (height + 1);
		break;

	case PANEL_APPLET_ORIENT_UP:
		y -= (req.height + 1);
		break;

	case PANEL_APPLET_ORIENT_LEFT:
		x -= req.width;
		break;

	case PANEL_APPLET_ORIENT_RIGHT:
		x += width;
		break;
	}

	gint screen_width = gdk_screen_width();
	gint screen_height = gdk_screen_height();
	x = CLAMP(x - 2, 0, MAX(0, screen_width - req.width));
	y = CLAMP(y - 2, 0, MAX(0, screen_height - req.height));

	gtk_window_move(GTK_WINDOW(g_win.window), x, y);
}



